//	udStFiLrSpectraMax
//	Andreas Wirsing
//
//////////////////***************//////////////////
//
//	DOCUMENTATION
//	
//	SpectraMaxDataLoader was developed for loading data of native SpectraMax binary files into Igor Pro.
//	At the time of development SpectraMas version was V 1.1d and Igor version was 5.05A.
//	It is not tested, whether also data of other versions of SpectraMax are loaded properly.
//	Because of the changed Note command, the procedure does not run in Igor prior to 5.05A.
//	
//	The loader requires the user-defined standardized file loader package provided by J. Weimer on igorexchange.com.
//	If renaming the loaded waves manually, an dialog is presented, where the user can - similary to the built-in load waves menu - enter wave names.
//	Default names are filename + 'Spx', where x is the number of spectrum (, + 'Rey', where y is the number of the region, if there's more than one).
//	Leaving a field blank skips loading of the particular spektrum as it is known from Igor's built-in load wave dialogue.
//	Pressing 'Cancel' aborts data loading for the particular file.
//	Name As "File Header Info" uses the above mentioned defaults and overwrites pre-existing waves.
//	Auto naming uses SpectraMaxN, where N is a number so that the resulting wave name is unique. This option is not recommended
//	as you will not know after loading which wave is related to which spectrum in the file.
//	The data loader reads first some parameters stored in the parameter fields within the file and writes them to parameter waves.
//	The parameters are tested if the values are meaningful. In the case, they are not, an warning message is printed to the history.
//	If an error is fatal, loading is aborted.
//	The waves are also scaled according to the parameters found in the file header.
//	At the end of this procedure file there is a table showing the mapping of various byte parameters, e. g. X_- and Y_value_identifier.
//
//	IMPORTANT: More than one data block per region may cause problems and is not supported.
//	(As a matter of fact, it is not known, what 'number of data blocks per region' in the SpectraMax binary files does mean.)
//
//
//	RELEASE NOTES
//
//	Version3.0
//	The SpectraMaxDataLoader became finally a user-defined standardized file loader. Some parts of the code have been re-written to be meet
//	the demands of a user-defined standardized file loader. There are still some improvements to be made...
//	added ModuleName pragma, functions are now static.
//	The "stand-alone" version series #2.x will probably not become enhanced any longer.
//
//	Version2.5
//	Added named window hook for panel 'Provide Wave Names' in order to be able to start data loading by pressing simply the return or enter key
//	and abort loading by pressing 'esc'.
//	Added version pragma.
//
//	Version2.4
//	Data readout in LoadDataFromFile() improved (could be done in one step without loop).
//
//	Version2.3
//	Improved handling of global variables and strings:
//	Using 'LoadSpectraMaxFiles()' sets 'S_fileName' to the name of the last loaded file, 'S_waveNames' to the list of all loaded wave names and
//	'V_flag' to the number of loaded waves. If one file coudn't be found and loading was aborted by the user, than 'V_flag' is set to zero
//	and 'S_fileName', 'S_waveNames' and 'S_path' are set to empty strings.
//
//	Version2.2
//	Enhanced functionality via the function 'LoadSpectraMaxFiles()' (plural), which lets the user easily load a series of SpectraMax files with progressional endnumbers.
//	In that case S_waveNames contains the names of all loaded waves, whereas the values of V_flag, S_filename reflect only the last file.
//	If 'Remember settings' is checked, global variables which store the settings are not killed.
//	'Use desault wave names' ckeckbox enabels or disables the ability to change the default wave names or skip single columns or whole files in the file range
//	via the usual dialog that is also shown when you call 'LoadSpectraMaxFile()' through the 'Data->Load Waves->Load SpectraMax file...' menu.
//
//	Version2.1
//	'SetScale d 0, 0, "counts", $wavenamestr' included, if 'block_unite'=52.
//	'unitstr="nm"' in 'SetScale /I x' inserted, if 'unite' of the region is 5 (nm).
//	V_flag, S_filename, S_path and S_waveNames are now set as in the Igor command 'LoadWave'.
//
//	Version2.0
//	three waves (SpectraMaxVariableWave, SpectraMaxTextWave and SpectraMaxPosWave) are now created to store the parameters associated with the file.
//	Interchange of these variables between the different functions is now easier. These three waves can be kept (checkbox in the control panel).
//	Files with more than one region are now loaded properly.
//	A bug has been fixed where a file with only 4 block parameter was not loaded correctly.
//	dynamic size of the 'Provide wave names' panel depending of the number of spectra
//	updated documenation
//
//	Version1.5
//	added documentation
//	added release notes
//	some minor bugs are fixed where parameters are not printed properly in the wave note
//
//////////////////***************//////////////////


#pragma rtGlobals=1		// Use modern global access method.
#pragma igorversion=6.03
#pragma version=3.0

#include "udFLStandardFunctions", versions>=2.0
#pragma ModuleName=udStFiLrSectraMax


// **********************************************
// Globals and Localizations

Static Constant kVersion = 1.0

// **********************************************
// Initialize (Zero + Query + Check for PlugIns)

Static Function udFLSpectraMaxInititalize(udFL)
	STRUCT udFLStandardStructure &udFL
	
	udFLZeroStructure(udFL)
	udFLSpectraMaxQuery(udFL)
	return 0
end

// **********************************************
// Query

Static Function udFLSpectraMaxQuery(udFL)
	STRUCT udFLStandardStructure &udFL
		
	udFL.mimetype = "????"
	udFL.extensions = ".L00;"
	udFL.loadModes += "Manual;"
	udFL.procModes += "Preserve ParamWaves"
	udFL.itemsType = 4			// returns waves
	udFL.itemsDim[p] = p*0
	udFL.itemsDim[0]=1			// waves are nominally scaled numeric
	udFL.setNames=(4+16)		// file header info and manually
	udFLSpectraMaxInitPlugIns(udFL)
	return 0	
end

// **********************************************
// Check for PlugIns

Static Function udFLSpectraMaxInitPlugIns(udFL)
	STRUCT udFLStandardStructure &udFL
	
//	udFLCheckForPlugIn("udFLPlugInLinkDisplay",udFL)
	return 0
end

// **********************************************
// File Loader

Function udStFiLrSpectraMax(udFL)
	STRUCT udFLStandardStructure &udFL
	
	variable refNum, ic, nt
	string pathStr, wavesLoaded, twList
	
	switch(udFL.userLCtrl)
		case -1:		// initialize		
			udFLSpectraMaxInititalize(udFL)		
			return 0
		case 0:		// query
			udFLSpectraMaxQuery(udFL)
			return 0
		case 1:		// auto
			if (strlen(udFL.fileList)==0)
				udFLSetError(udFL,"No files in the selection list!",-1)
				return -1
			endif
			break
		case 2:		// manual
			Open/D/Z=2/M="Looking for a SpectraMax file"/R/T=".L00" refnum
			if (strlen(S_filename)!=0)
				udFL.filelist = S_filename + ";"
			else
				return -1
			endif
			break
	endswitch
	
	if (udFL.reportCtrl >= 1)
		print "LOADING SPECTRAMAX DATA"
		print "... assuring file paths"
	endif
	if (udFLAssureFilePaths(udFL)<0)
		return -1
	endif
	if (strlen(udFL.pathStr)!=0)
		pathStr = udFL.pathStr
		NewPath/O/Q tmpSpectraMaxdir, pathStr
	endif
	
	if (udFL.reportCtrl >= 1)
		print "... setting data folder location"
	endif
	string cdf = GetDataFolder(1)
	udFLSetDataFolder(udFL)
	
	if (udFL.reportCtrl >= 1)
		print "... loading"
	endif
		
	nt = ItemsInList(udFL.fileList)
	for (ic=0;ic<nt;ic+=1)
			
		WavesLoaded = LoadSpectraMaxFile(ic,udFL)
		
		if (strlen(WavesLoaded)!=0)
				
			WavesLoaded = ProcessSpectraMaxWaves(WavesLoaded, udFL)
			WavesLoaded = RenameSpectraMaxWaves(ic,WavesLoaded, udFL)
//			WavesLoaded = DoUserDataSpectraMaxWaves(WavesLoaded, udFL)
//			LinkSpectraMaxWaves(WavesLoaded,udFL)
			udFL.itLst.WAVES += WavesLoaded
				
		endif
		
	endfor


	udFL.itemsCount = ItemsInList(udFL.itLst.WAVES)

	ProcessSpectraMaxWaves(udFL.itLst.Waves,udFL)

	SetDataFolder $cdf

	if (strlen(udFL.pathStr)!=0)
		KillPath tmpSpectraMaxDir
	endif

	if (udFL.reportCtrl == 2)
		print "Structure @ Loader:", udFL
	endif

	return 0
end

Static Function/S LoadSpectraMaxFile(ic, udFL)
	variable ic
	STRUCT udFLStandardStructure &udFL

	string fileStr=StringFromList(ic,udFL.fileList)
	variable file_refnum

	if (udFL.reportCtrl >= 1)
		print "... file ", fileStr
	endif
	if (strlen(udFL.pathStr)==0)
		Open/r file_refNum fileStr
	else
		Open/r/P=tmpSpectraMaxdir file_refNum fileStr
	endif

	variable i, j, k, l, m, v=0, w, file_variable, spectrum_variable, region_variable, block_variable, start, notevar
	string file_string, spectrum_string, region_string, block_string, wavename0, alertstrTYPE_DONNEES, Y_alertstr, notestr="", unitstr, S_note=""
	make /O/T/N=500 SpectraMaxTextWave=""; wave/t wt=SpectraMaxTextWave
	make /O/N=500 SpectraMaxVariableWave; wave wv=SpectraMaxVariableWave
	i=wv[451]; wv=NAN; wv[451]=i		// preserves CheckBox0 Status
	make /O/N=500 SpectraMaxPosWave=NAN; wave wp=SpectraMaxPosWave
	wt[450]="Cancel Status"; wt[451]="Status Checkbox0"; wt[452]="First Data Point"
	wt[453]="file_refnum"; wv[453]=file_refnum
	FStatus file_refnum
	if(udFl.reportCtrl>1)
		printf "\rparameter:\r"
		printf "V_logEOF=%d, \r" V_logEOF
	endif
///////		File Header:
	wt[v]="file header:"; v+=4
	FSetPos file_refnum, 52
	string TYPE_ENR; FReadLine /N=1 file_refnum, TYPE_ENR; wt[v]=TYPE_ENR; wp[v]=52; v+=1
	variable taille_parametres; FBinRead /B=3 /F=2 /U file_refnum, taille_parametres; wt[v]="taille_parametres"
	wv[v]=taille_parametres; wp[v]=wp[(v-1)]+1; v+=1
	variable NBRE_SPECTRES; FBinRead /B=3 /F=2 /U file_refnum, NBRE_SPECTRES; wt[v]="NBRE_SPECTRES"
	wv[v]=NBRE_SPECTRES; wp[v]=wp[(v-1)]+2; v+=1
	variable TAILLE_SPECTRE; FBinRead /B=3 /F=3 /U file_refnum, TAILLE_SPECTRE; wt[v]="TAILLE_SPECTRE"
	wv[v]=TAILLE_SPECTRE; wp[v]=wp[(v-1)]+2; v+=1
	if(udFl.reportCtrl>1)
		printf "TYPE_ENR=%s, taille_parametres=%d, NBRE_SPECTRES=%d, TAILLE_SPECTRE=%d\r" TYPE_ENR, taille_parametres, NBRE_SPECTRES, TAILLE_SPECTRE
	endif
	variable ENR_DONNEES; FBinRead /B=3 /F=2 /U file_refnum, ENR_DONNEES; wt[v]="ENR_DONNEES"
	wv[v]=ENR_DONNEES; wp[v]=wp[(v-1)]+4; wp[v]=wp[(v-1)]+4; v+=1
	variable TAILLE_DONNEES; FBinRead /B=3 /F=3 /U file_refnum, TAILLE_DONNEES; wt[v]="TAILLE_DONNEES"
	wv[v]=TAILLE_DONNEES; wp[v]=wp[(v-1)]+2; wp[v]=wp[(v-1)]+2; v+=1
	variable PARAM_REGIONS; FBinRead /B=3 /F=1 /U file_refnum, PARAM_REGIONS; wt[v]="PARAM_REGIONS"
	wv[v]=PARAM_REGIONS; wp[v]=wp[(v-1)]+4; wp[v]=wp[(v-1)]+4; v+=1
	variable ENR_REGIONS; FBinRead /B=3 /F=3 /U file_refnum, ENR_REGIONS; wt[v]="ENR_REGIONS"
	wv[v]=ENR_REGIONS; wp[v]=wp[(v-1)]+1; wp[v]=wp[(v-1)]+1; v+=1
	if(udFl.reportCtrl>1)
		printf "ENR_DONNEES=%d, TAILLE_DONNEES=%d, PARAM_REGIONS=%d, ENR_REGIONS=%d\r" ENR_DONNEES, TAILLE_DONNEES, PARAM_REGIONS, ENR_REGIONS
	endif
	variable NBRE_REGIONS; FBinRead /B=3 /F=1 /U file_refnum, NBRE_REGIONS; wt[v]="NBRE_REGIONS"
	wv[v]=NBRE_REGIONS; wp[v]=wp[(v-1)]+4; v+=1
	variable TAILLE_REGIONS; FBinRead /B=3 /F=2 /U file_refnum, TAILLE_REGIONS; wt[v]="TAILLE_REGIONS"
	wv[v]=TAILLE_REGIONS; wp[v]=wp[(v-1)]+1; v+=1
	variable ENR_PICS; FBinRead /B=3 /F=3 /U file_refnum, ENR_PICS; wt[v]="ENR_PICS"
	wv[v]=ENR_PICS; wp[v]=wp[(v-1)]+2; v+=1
	variable TAILLE_PICS; FBinRead /B=3 /F=2 /U file_refnum, TAILLE_PICS
	wt[v]="TAILLE_PICS"; wv[v]=TAILLE_PICS; wp[v]=wp[(v-1)]+4; v+=1
	if(udFl.reportCtrl>1)
		printf "NBRE_REGIONS=%d, TAILLE_REGIONS=%d, ENR_PICS=%d, TAILLE_PICS=%d\r" NBRE_REGIONS, TAILLE_REGIONS, ENR_PICS, TAILLE_PICS
	endif
	variable ENR_COMMENTAIRES; FBinRead /B=3 /F=3 /U file_refnum, ENR_COMMENTAIRES
	wt[v]="ENR_COMMENTAIRES"; wv[v]=ENR_COMMENTAIRES; wp[v]=wp[(v-1)]+2; v+=1
	variable TAILLE_COMMENTAIRES; FBinRead /B=3 /F=2 /U file_refnum, TAILLE_COMMENTAIRES
	wt[v]="TAILLE_COMMENTAIRES"; wv[v]=TAILLE_COMMENTAIRES; wp[v]=wp[(v-1)]+4; v+=1
	variable NB_VARS_PAR; FBinRead /B=3 /F=1 /U file_refnum, NB_VARS_PAR
	wt[v]="NB_VARS_PAR"; wv[v]=NB_VARS_PAR; wp[v]=wp[(v-1)]+2; v+=1
	variable NB_CHAINES_PAR; FBinRead /B=3 /F=1 /U file_refnum, NB_CHAINES_PAR
	wt[v]="NB_CHAINES_PAR"; wv[v]=NB_CHAINES_PAR; wp[v]=wp[(v-1)]+1; v+=1
	if(udFl.reportCtrl>1)
		printf "ENR_COMMENTAIRES=%d, TAILLE_COMMENTAIRES=%d, NB_VARS_PAR=%d, NB_CHAINES_PAR=%d\r" ENR_COMMENTAIRES, TAILLE_COMMENTAIRES, NB_VARS_PAR, NB_CHAINES_PAR
	endif
	variable NB_VARS_SPECTRE; FBinRead /B=3 /F=1 /U file_refnum, NB_VARS_SPECTRE
	wt[v]="NB_VARS_SPECTRE"; wv[v]=NB_VARS_SPECTRE; wp[v]=wp[(v-1)]+1; v+=1
	variable NB_CHAINES_SPECTRE; FBinRead /B=3 /F=1 /U file_refnum, NB_CHAINES_SPECTRE
	wt[v]="NB_CHAINES_SPECTRE"; wv[v]=NB_CHAINES_SPECTRE; wp[v]=wp[(v-1)]+1; v+=1
	variable NB_VARS_REGION; FBinRead /B=3 /F=1 /U file_refnum, NB_VARS_REGION
	wt[v]="NB_VARS_REGION"; wv[v]=NB_VARS_REGION; wp[v]=wp[(v-1)]+1; v+=1
	variable NB_CHAINES_REGION; FBinRead /B=3 /F=1 /U file_refnum, NB_CHAINES_REGION
	wt[v]="NB_CHAINES_REGION"; wv[v]=NB_CHAINES_REGION; wp[v]=wp[(v-1)]+1; v+=1
	if(udFl.reportCtrl>1)
		printf "NB_VARS_SPECTRE=%d, NB_CHAINES_SPECTRE=%d, NB_VARS_REGION=%d, NB_CHAINES_REGION=%d\r" NB_VARS_SPECTRE, NB_CHAINES_SPECTRE, NB_VARS_REGION, NB_CHAINES_REGION
	endif
	variable NB_VARS_BLOC; FBinRead /B=3 /F=1 /U file_refnum, NB_VARS_BLOC
	wt[v]="NB_VARS_BLOC"; wv[v]=NB_VARS_BLOC; wp[v]=wp[(v-1)]+1; v+=1
	variable NB_CHAINES_BLOC; FBinRead /B=3 /F=1 /U file_refnum, NB_CHAINES_BLOC
	wt[v]="NB_CHAINES_BLOC"; wv[v]=NB_CHAINES_BLOC; wp[v]=wp[(v-1)]+1; v+=1
	variable ENR_SUPP; FBinRead /B=3 /F=3 /U file_refnum, ENR_SUPP
	wt[v]="ENR_SUPP"; wv[v]=ENR_SUPP; wp[v]=wp[(v-1)]+1; v+=1
	variable TAILLE_ENR_SUPP; FBinRead /B=3 /F=2 /U file_refnum, TAILLE_ENR_SUPP
	wt[v]="TAILLE_ENR_SUPP"; wv[v]=TAILLE_ENR_SUPP; wp[v]=wp[(v-1)]+4; v+=1
	if(udFl.reportCtrl>1)
		printf "NB_VARS_BLOC=%d, NB_CHAINES_BLOC=%d, ENR_SUPP=%d, TAILLE_ENR_SUPP=%d, \r" NB_VARS_BLOC, NB_CHAINES_BLOC, ENR_SUPP, TAILLE_ENR_SUPP
	endif
	variable ENTETE_2; FBinRead /B=3 /F=3 /U file_refnum, ENTETE_2
	wt[v]="ENTETE_2"; wv[v]=ENTETE_2; wp[v]=wp[(v-1)]+2; v+=1
	variable TAILLE_ENTETE; FBinRead /B=3 /F=2 /U file_refnum, TAILLE_ENTETE
	wt[v]="TAILLE_ENTETE"; wv[v]=TAILLE_ENTETE; wp[v]=wp[(v-1)]+4; v+=1
	variable taille_fichier; FBinRead /B=3 /F=3 /U file_refnum, taille_fichier
	wt[v]="taille_fichier"; wv[v]=taille_fichier; wp[v]=wp[(v-1)]+2
	w=v; v=40
	if(udFl.reportCtrl>1)
		printf "ENTETE_2=%d, TAILLE_ENTETE=%d, taille_fichier=%d\r" ENTETE_2, TAILLE_ENTETE, taille_fichier
	endif
///////		File Variables:
	wt[v]="file variables:"; v+=2
	wp[v]=wp[w]+4
	for(i=1; i<=NB_VARS_PAR; i+=1)
		FBinRead /B=3 /F=4 file_refnum, file_variable
		wt[v]="file variable " + num2str(i); wv[v]=file_variable; v+=1; wp[v]=wp[(v-1)]+4
		if(udFl.reportCtrl>1)
			printf "file_variable%d=%g\r" i, file_variable
		endif
	endfor
	wp[v]=NAN; v+=1; wp[v]=wp[(v-2)]+5
	for(i=1; i<=NB_CHAINES_PAR; i+=1)
		FStatus file_refnum; FSetPos file_refnum, (V_filePos+1); FStatus file_refnum
		FReadLine /N=20 file_refnum, file_string
		FSetPos file_refnum, (V_filePos+20)
		wt[v]=file_string; v+=1; wp[v]=wp[(v-1)]+21
		if(udFl.reportCtrl>1)
			printf "file_string%d=%s, V_filePos=%d\r" i, file_string, V_filePos
		endif
		if(i==3)
			string DATE_CHAINE = file_string
			do
				if(strlen(DATE_CHAINE)<=14)
					break
				endif
				DATE_CHAINE = RemoveEnding(DATE_CHAINE)
			while(1)		
		endif
	endfor
	wp[v]=NAN; w=v
///////		Start Data Block
///////		Spectrum Variables of Spectrum 1:
	v=70; wt[v]="spectrum variables of spectrum 1:"; v+=2; wp[v]=wp[(w-1)]+20
	for(i=1; i<=NB_VARS_SPECTRE; i+=1)
		FBinRead /B=3 /F=4 file_refnum, spectrum_variable
		wt[v]="spectrum variable " + num2str(i); wv[v]=file_variable; v+=1; wp[v]=wp[(v-1)]+4
		if(udFl.reportCtrl>1)
			printf "spectrum_variable%d=%g\r" i, spectrum_variable
		endif
	endfor
	wp[v]=NAN; v+=1; wp[v]=wp[(v-2)]+5
	for(i=1; i<=NB_CHAINES_SPECTRE; i+=1)
		FStatus file_refnum; FSetPos file_refnum, (V_filePos+1)
		FReadLine /N=20 file_refnum, spectrum_string; FStatus file_refnum
		wt[v]=file_string; v+=1; wp[v]=wp[(v-1)]+21
		if(udFl.reportCtrl>1)
			printf "spectrum_string%d=%s, V_filePos=%d\r" i, spectrum_string, V_filePos
		endif
	endfor
	wp[v]=NAN; w=v; v=90
	wt[v]="region variables of spectrum 1:"; v+=2; wp[v]=wp[(w-1)]+20
///////		Regions Variables of Spectrum 1:
	for(i=1; i<=NB_VARS_REGION; i+=1)
		FBinRead /B=3 /F=4 file_refnum, region_variable
		wt[v]="region variable " + num2str(i); wv[v]=region_variable; v+=1; wp[v]=wp[(v-1)]+4
		if(udFl.reportCtrl>1)
			printf "region_variable%d=%g\r" i, region_variable
		endif
	endfor
	wp[v]=NAN; v+=1; wp[v]=wp[(v-2)]+5
	for(i=1; i<=NB_CHAINES_REGION; i+=1)
		FStatus file_refnum; FSetPos file_refnum, (V_filePos+1)
		FReadLine /N=20 file_refnum, region_string; FStatus file_refnum
		wt[v]=region_string; v+=1; wp[v]=wp[(v-1)]+21
		if(udFl.reportCtrl>1)
			printf "region_string%d=%s, V_filePos=%d\r" i, region_string, V_filePos
		endif
	endfor
	wp[v]=NAN; w=v
	v=130; wt[v]="first block header of spectrum 1:"; v+=2
	FBinRead /B=3 /F=1 file_refnum, i; wt[v]="block_unite"; wv[v]=i;  wp[v]=wp[(w-1)]+20; v+=1
	FBinRead /B=3 /F=4 file_refnum, i; wt[v]="block_mini"; wv[v]=i;  wp[v]=wp[(v-1)]+1; v+=1
	FBinRead /B=3 /F=4 file_refnum, i; wt[v]="block_maxi"; wv[v]=i;  wp[v]=wp[(v-1)]+4; w=v
	wv[452]=wp[v]+4
///////		Block Variables:
	v=140; wt[v]="block variables of spectrum 1:"; v+=2; wp[v]=wp[(w)]+4
	for(i=1; i<=NB_VARS_BLOC; i+=1)
		FBinRead /B=3 /F=4 file_refnum, block_variable
		wt[v]="block variable " + num2str(i); wv[v]=block_variable; v+=1; wp[v]=wp[(v-1)]+4
		if(udFl.reportCtrl>1)
			printf "block_variable%d=%g\r" i, block_variable
		endif
	endfor
	wp[v]=NAN; v+=1; wp[v]=wp[(v-2)]+5
	for(i=1; i<=NB_CHAINES_BLOC; i+=1)
		FStatus file_refnum; FSetPos file_refnum, (V_filePos+1)
		FReadLine /N=20 file_refnum, block_string; FStatus file_refnum
		wt[v]=region_string; v+=1; wp[v]=wp[(v-1)]+21
		if(udFl.reportCtrl>1)
			printf "block_string%d=%s, V_filePos=%d\r" i, block_string, V_filePos
		endif
		if(i==NB_CHAINES_BLOC)
			wv[452]=wp[v]-1; wp[v]=NAN
		endif
	endfor
////////		Region Header Block
	v=160; wt[v]="region header:"; v+=2; wp[v]=wv[11]
	if(PARAM_REGIONS==0&&NBRE_REGIONS>1)
		udFLSetError(udFL,"Data loading will be aborted, because important parameter positions cannot be located!",-1)
		close file_refnum
		udFLHandleError(udFL)
	else
		FSetPos file_refnum, (ENR_REGIONS)
		for(i=1; i<=(TAILLE_REGIONS/19); i+=1)
			variable LONGUEUR; FBinRead /B=3 /F=2 file_refnum, LONGUEUR		// not used
			wt[v]="LONGUEUR "+num2str(i); wv[v]=LONGUEUR; v+=1; wp[v]=wp[(v-1)]+2
			variable TYPE_DONNEES; FBinRead /B=3 /F=1 file_refnum, TYPE_DONNEES		// data type, should be 6
			wt[v]="TYPE_DONNEES "+num2str(i); wv[v]=TYPE_DONNEES; v+=1; wp[v]=wp[(v-1)]+1
			if(TYPE_DONNEES!=6)		// This is an region header block variable which specifies the data type.
				wt[460+i]= "Variable TYPE_DONNEES of region block "+num2str(i)+" is supposed to be 6! In this region block it is set to " + num2str(TYPE_DONNEES)
				udFLSetError(udFL,wt[460+i],0)
				udFLHandleError(udFL)
			endif
			variable NBRE_BLOCS; FBinRead /B=3 /F=1 file_refnum, NBRE_BLOCS
			wt[v]="NBRE_BLOCS "+num2str(i); wv[v]=NBRE_BLOCS; v+=1; wp[v]=wp[(v-1)]+1
			variable unite; FBinRead /B=3 /F=1 file_refnum, unite
			wt[v]="unite "+num2str(i); wv[v]=unite; v+=1; wp[v]=wp[(v-1)]+1
			variable VALXMINI; FBinRead /B=3 /F=4 file_refnum, VALXMINI
			wt[v]="VALXMINI "+num2str(i); wv[v]=VALXMINI; v+=1; wp[v]=wp[(v-1)]+4
			variable VALXMAXI; FBinRead /B=3 /F=4 file_refnum, VALXMAXI
			wt[v]="VALXMAXI "+num2str(i); wv[v]=VALXMAXI; v+=1; wp[v]=wp[(v-1)]+4
			variable INCREMENT; FBinRead /B=3 /F=4 file_refnum, INCREMENT
			wt[v]="INCREMENT "+num2str(i); wv[v]=INCREMENT; v+=1; wp[v]=wp[(v-1)]+4
			variable NBRE_POINTS; FBinRead /B=3 /F=2 file_refnum, NBRE_POINTS
			wt[v]="NBRE_POINTS "+num2str(i); wv[v]=NBRE_POINTS; v+=1; wp[v]=wp[(v-1)]+2
			if(udFl.reportCtrl>1)
				printf "LONGUEUR=%d, TYPE_DONNEES=%d, NBRE_BLOCS=%d, unite=%d, VALXMINI=%g, VALXMAXI=%g, INCREMENT=%g, NBRE_POINTS=%d\r\r" LONGUEUR, TYPE_DONNEES, NBRE_BLOCS, unite, VALXMINI, VALXMAXI, INCREMENT, NBRE_POINTS
			endif
		endfor
	endif
	wp[v]=NAN; v+=1
///////		Tests, if variables has been properly read out
	if(taille_parametres!=ENR_DONNEES)
		udFLSetError(udFL,"Variables taille_parametres and ENR_DONNEES should have the same value, but they do not!",0)
		udFLHandleError(udFL)
	endif
	if(ENR_REGIONS!=TAILLE_DONNEES + ENR_DONNEES)
		udFLSetError(udFL,"Variable ENR_REGIONS should be equal to TAILLE_DONNEES + ENR_DONNEES, but it does not! ",0)
		udFLHandleError(udFL)
	endif
	if(TAILLE_DONNEES!=NBRE_SPECTRES * TAILLE_SPECTRE)
		udFLSetError(udFL,"Variable TAILLE_DONNEES should be equal to NBRE_SPECTRES * TAILLE_SPECTRE, but it does not!",0)
		udFLHandleError(udFL)
	endif
	if(taille_parametres!=ENR_DONNEES)
		udFLSetError(udFL,"This procedure was developed to load data with only 1 region block per spectrum. This file contains spectra with more than 1 region block, which may have caused misbehaviour!",0)
		udFLHandleError(udFL)
	endif
	if(!(PARAM_REGIONS==1)&&NBRE_REGIONS>1)
		udFLSetError(udFL,"Data loading will be aborted, because important parameter positions cannot be located!",-1)
		close file_refnum
		udFLHandleError(udFL)
	endif
////////		Provide Wavenames
	wavename0=RemoveEnding(S_fileName, ".L00")
	j=1; k=1
	for(i=1; i<=(NBRE_SPECTRES*NBRE_REGIONS); i+=1)
		if(NBRE_REGIONS>1)
			wt[(300+i)]=wavename0+"_Sp"+num2str(j)+"_Re"+num2str(k)
		else
			wt[(300+i)]=wavename0+"_Sp"+num2str(j)
		endif
		k+=1
		if(k==(NBRE_REGIONS+1))
			j+=1
			k=1
		endif
	endfor
	if(udFL.setNames==16)
		MakeWaveNamePanel(NBRE_SPECTRES*NBRE_REGIONS)
		PauseForUser Paneltobekilledlater, Spectramaxreadmepanel
		if(wv[451]==1)
			udFL.setNames=4		// may not be conform with how a udStFiLr should be designed, because it switches between naming options inside the loading procedure
		endif
	endif
////////		make waves
	string S_waveNamesLoc=""
	if(wv[450]!=1)		// ==1, if 'Cancel' was pressed in MakeWaveNamePanel
		j=1; k=0; l=0; m=0
		for(i=1; i<=(NBRE_SPECTRES*NBRE_REGIONS); i+=1)
			start=wv[452]+l*TAILLE_SPECTRE+k
			NBRE_POINTS=wv[(169+(j-1)*8)]
			if(!stringmatch(wt[(300+i)],"_skip column_")==1)
				// data loading:
				make/o/n=(NBRE_POINTS) $wt[(300+i)]; wave w_out=$wt[(300+i)]
				fsetpos file_refnum,start
				fbinread/b=3 file_refnum, w_out	// w_out must be single precision float
				S_waveNamesLoc+=wt[(300+i)]+";"
				if(wv[(165+(NBRE_REGIONS-1)*8)]==5)
					unitstr="nm"
				else
					unitstr=""
				endif
				if(PARAM_REGIONS==1)
					SetScale /I x (wv[166+(j-1)*8]), (wv[167+(j-1)*8]), unitstr, w_out
				else
					SetScale /I x (wv[166+m*8]), (wv[167+m*8]), unitstr, w_out
				endif
				if(wv[132]==52)
					SetScale d 0, 0, "counts", w_out
				endif
				S_note=""
				sprintf S_note,"%sdate=%s;", S_note,DATE_CHAINE
				notestr=""; notestr=padstring(notestr,12,0x20)
				FSetPos file_refnum, (wp[78]+l*TAILLE_SPECTRE); FBinRead /B=3 file_refnum, notestr
				FSetPos file_refnum, (wp[72]); FBinRead /B=3 /F=4 file_refnum, notevar
				if(notevar>0)
					sprintf S_note,"%s%s=%g;", S_note,notestr,notevar
				endif
				notestr=""; notestr=padstring(notestr,18,0x20)
				FSetPos file_refnum, (wp[105]+l*TAILLE_SPECTRE+k); FBinRead file_refnum, notestr
				FSetPos file_refnum, (wp[92]+l*TAILLE_SPECTRE+k); FBinRead /B=3 /F=4 file_refnum, notevar
				sprintf S_note,"%s%s=%g;", S_note,notestr,notevar
				notestr=""; notestr=padstring(notestr,20,0x20)
				FSetPos file_refnum, (wp[106]+l*TAILLE_SPECTRE+k); FBinRead file_refnum, notestr
				FSetPos file_refnum, (wp[93]+l*TAILLE_SPECTRE+k); FBinRead /B=3 /F=4 file_refnum, notevar
				sprintf S_note,"%s%s=%g;", S_note,notestr,notevar
				notestr=""; notestr=padstring(notestr,16,0x20)
				FSetPos file_refnum, (wp[108]+l*TAILLE_SPECTRE+k); FBinRead file_refnum, notestr
				FSetPos file_refnum, (wp[95]+l*TAILLE_SPECTRE+k); FBinRead /B=3 /F=4 file_refnum, notevar
				if(notevar>0&&stringmatch(notestr,"!*chaine*"))
					sprintf S_note,"%s%s=%g;", S_note,notestr,notevar
				endif
				notestr="";notestr=padstring(notestr,11,0x20)
				FSetPos file_refnum, (wp[110]+l*TAILLE_SPECTRE+k); FBinRead file_refnum, notestr
				FSetPos file_refnum, (wp[97]+l*TAILLE_SPECTRE+k); FBinRead /B=3 /F=4 file_refnum, notevar
				sprintf S_note,"%s%s=%g;", S_note,notestr,notevar
				notestr=""; notestr=padstring(notestr,10,0x20)
				FSetPos file_refnum, (wp[111]+l*TAILLE_SPECTRE+k); FBinRead file_refnum, notestr
				FSetPos file_refnum, (wp[98]+l*TAILLE_SPECTRE+k); FBinRead /B=3 /F=4 file_refnum, notevar
				sprintf S_note,"%s%s=%g;", S_note,notestr,notevar
				FSetPos file_refnum, (wp[133]+l*TAILLE_SPECTRE+k)
				FBinRead /B=3 /F=4 file_refnum, notevar
				sprintf S_note,"%sV_min=%.8g;", S_note,notevar
				FBinRead /B=3 /F=4 file_refnum, notevar
				sprintf S_note,"%sV_max=%.8g;", S_note,notevar
				FSetPos file_refnum, (wp[132]+l*TAILLE_SPECTRE+k); FBinRead /B=3 /F=1 file_refnum, notevar
				if(notevar==52)
					notestr="(counts)"
				else
					notestr="(NOT 'counts')"
					Y_alertstr = "Y-value identifier of spectrum "+num2str(i)+" does not show \"counts\"!"
					udFLSetError(udFL,Y_alertstr,0)
					udFLHandleError(udFL)
				endif
				sprintf S_note,"%sY_value_identifier %s=%d;", S_note,notestr,notevar
				if(PARAM_REGIONS==1)
					if((wv[165+(j-1)*8])==5)
						notestr="(nm)"
					else
						notestr="(NOT 'nm')"
					endif
					sprintf S_note,"%sX_value_identifier %s=%d;", S_note,notestr,wv[165+(j-1)*8]
					sprintf S_note,"%sVALXMINI=%.8g;VALXMAXI=%.8g;INCREMENT=%.8g;NBRE_POINTS=%d;", S_note,wv[166+(j-1)*8],wv[167+(j-1)*8],wv[168+(j-1)*8],wv[169+(j-1)*8]
					if(TAILLE_COMMENTAIRES>0)
						FSetPos file_refnum, (ENR_COMMENTAIRES);notestr="";notestr=padstring(notestr,TAILLE_COMMENTAIRES,0x20);FBinRead /B file_refnum, notestr
						sprintf S_note,"%sCOMMENT=%s;", S_note,notestr
					endif
				else
					if((wv[165+m*8])==5)
						notestr="(nm)"
					else
						notestr="(NOT 'nm')"
					endif
					sprintf S_note,"%sX_value_identifier %s=%d;", S_note,notestr,wv[165+m*8]
					sprintf S_note,"%sVALXMINI=%.8g,VALXMAXI=%.8g;INCREMENT=%.8g;NBRE_POINTS=%d;" S_note,wv[166+m*8],wv[167+m*8],wv[168+m*8],wv[169+m*8]
					if(TAILLE_COMMENTAIRES>0)
						FSetPos file_refnum, (ENR_COMMENTAIRES);notestr="";notestr=padstring(notestr,TAILLE_COMMENTAIRES,0x20);FBinRead /B file_refnum, notestr
						sprintf S_note, "%sCOMMENT=%s;" S_note,notestr
					endif						
				endif
				note/k w_out,S_note
			endif		// (!stringmatch(wavenamestr1,"_skip column_")==1)
			k+=NBRE_POINTS*4+359		// ; print "k = "+num2str(k)
			if(j==(NBRE_REGIONS))
				j=0; k=0; l+=1
			endif
			j+=1; m+=1
		endfor		// (i=1; i<=(NBRE_SPECTRES*NBRE_REGIONS); i+=1)
	endif		// if(wv[450]!=1)		// wv[450]==1, if 'Cancel' was pressed in MakeWaveNamePanel
	close file_refnum
	return S_waveNamesLoc
//	if(wv[451]==0)		//has to be built in a Processing function
//		KillWaves /Z wv wp wt
//	endif
End


end

Static Function/S ProcessSpectraMaxWaves(WavesLoaded,udFL)
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL

	variable ic, nt
	string theOne

	if(udFL.userPCtrl!=2)
		killwaves/z SpectraMaxTextWave,SpectraMaxVariableWave,SpectraMaxPosWave
	endif
	
	return WavesLoaded

end

// **********************************************
// Apply User Data to Waves
//
// [...] wave selection

Static Function/S DoUserDataSpectraMaxWaves(WavesLoaded, udFL)
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL	

	return WavesLoaded
end

// **********************************************
// Rename Waves

Static Function/S RenameSpectraMaxWaves(ic,WavesLoaded,udFL)
	variable ic
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL

	switch(udFL.setNames)
		case 1:
			udFLSetError(udFL,"Because of the meaningless names auto naming is NOT recommended!",0)
			udFLHandleError(udFL)
			WavesLoaded=udFLRenameInput(WavesLoaded,"SpectraMaxWave")
			break
		case 4:
			// do nothing, because wave names are already set by the loader
			if (udFL.reportCtrl >= 1)
				print "waves are named by internal file structure"
			endif
			break
		case 16:
			// do nothing, because wave names are already set by the loader
			if (udFL.reportCtrl >= 1)
				print "waves are named by user"
			endif
			break
	endswitch
	return WavesLoaded
end

// **********************************************
// (possibly) Link the Waves

Static Function LinkSpectraMaxWaves(WavesLoaded,udFL)
	string WavesLoaded
	STRUCT udFLStandardStructure &udFL

	return 0
end

Static function MakeWaveNamePanel(a)
	variable a
	wave /T wt=SpectraMaxTextWave; wave wv=SpectraMaxVariableWave
	if(!(wv[451]==1|wv[451]==0))
		wv[451]=0
	endif
	variable b=trunc((a-1)/4), i, j=0, k=0, c=0
	string SetVarStr, TitleBoxStr, TitleBoxNameStr
	NewPanel /K=2/N=Paneltobekilledlater/W=(159,128,597,(226+b*48)) as "Provide wave names"
	for(i=301; i<(301+a); i+=1)
		j+=1; SetVarStr = "wavename"+num2str(j); TitleBoxStr="title"+num2str(j); TitleBoxNameStr="Wavename "+num2str(j)		//do not change, important for other functions!!
		SetVariable $SetVarStr,pos={(4+k*110),(2+c*48)},size={100,15},disable=0,title=" ",fSize=10,value= wt[i],proc=udStFiLrSectraMax#Do_skipColumn_
		TitleBox $TitleBoxStr,pos={(6+k*110),(22+c*48)},size={77,15},disable=2,title=TitleBoxNameStr,fSize=12,frame=0
		k+=1
		if(k==4)
			k=0; c+=1
		endif
	endfor
	Button load_button,pos={179,(47+b*48)},size={75,20},title="Load",proc=udStFiLrSectraMax#DoSetWavenamesAndKill
	Button cancel_button,pos={92,(47+b*48)},size={75,20},title="Cancel",proc=udStFiLrSectraMax#DoKillWithoutSetWavenames
	Button readMe_button,pos={6,(47+b*48)},size={75,20},title="Read Me...",proc=udStFiLrSectraMax#DoShowReadMe
	TitleBox version,pos={292,(47+b*48)},size={116,32},title="Version 3.0 (Aug. 2008)\rAndreas Wirsing",labelBack=(3,52428,1),fSize=9,frame=2
	CheckBox checkbox0,pos={5,(72+b*48)},size={130,14},value=0,title="name following files by file header info",fSize=10,proc=udStFiLrSectraMax#ControlParameterCheckBox
	SetWindow kwTopWin, hook(PaneltobekilledlaterHook)=udStFiLrSectraMax#F_PaneltobekilledlaterHook
	if(wv[451]==0)
		CheckBox checkbox0,value=0
	else
		CheckBox checkbox0,value=1
	endif
End

Static Function Do_skipColumn_(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName
	Variable varNum	// value of variable as number
	String varStr		// value of variable as string
	String varName		// name of variable
	variable a=strlen(varname)
	string b=varname[(a-4)]+varname[(a-3)]+varname[(a-2)]
	a=str2num(b)
	wave/t wt=SpectraMaxTextwave
	if(stringmatch(varStr,"")==1)
		wt[a]="_skip column_"
	endif
End

Static function DoSetWaveNamesAndKill(load_button)	:	ButtonControl
	string load_button
	wave/t wt=SpectraMaxTexwtave; wave wv=SpectraMaxVariableWave
	variable a=wv[4]*wv[10], b, i, j
	for(i=301; i<=(301+a); i+=1)
		for(j=(i+1); j<=(301+a); j+=1)
			b=stringmatch(wt[i],wt[j])
			if(b==1&&stringmatch(wt[i],"_skip column_")==0)
				DoAlert 0, "two different waves must not have the same name!\r\rChange at least 1 wavename!"
				return 0
			endif
		endfor
	endfor
	ControlInfo /W=Paneltobekilledlater ValDisplay
	DoWindow /K Paneltobekilledlater
End

Static function DoKillWithoutSetWavenames(cancel_button)	:	Buttoncontrol
	string cancel_button
	DoWindow /K Paneltobekilledlater
	wave wv=SpectraMaxVariableWave; wv[450]=1
End

Static function DoShowReadMe(readMe_button)
	string readMe_button
	DoWindow Spectramaxreadmepanel
	if(V_flag==1)
		DoWindow /F Spectramaxreadmepanel
	else
		NewPanel /N=Spectramaxreadmepanel/W=(330,272,650,433) as "Read me"
		TitleBox title0,pos={26,10},size={237,12},title="This file was developed to load data recorded with"
		TitleBox title0,fSize=10,frame=0
		TitleBox title0_1,pos={26,25},size={227,12},title="SpectraMax Software (Copyright: Jobin-Yvon,"
		TitleBox title0_1,fSize=10,frame=0
		Button load_button,pos={123,113},size={76,20},title="Close",proc=udStFiLrSectraMax#DoClose_Spectramaxreadmepanel
		TitleBox title0_4,pos={26,40},size={132,12},title="Version 1.1d, 27/06/95)."
		TitleBox title0_4,fSize=10,frame=0
		TitleBox title0_5,pos={26,55},size={248,12},title="Other versions have not been tested, but may work."
		TitleBox title0_5,fSize=10,frame=0
		TitleBox title0_2,pos={26,70},size={269,12},title="For documentation, see the header of the procedure file."
		TitleBox title0_2,fSize=10,frame=0
	endif
End

Static function DoClose_Spectramaxreadmepanel(load_button)
	string load_button
	DoWindow /K Spectramaxreadmepanel
End


Static function ControlParameterCheckBox(name,value) : CheckBoxControl
	string name
	variable value
	wave wv=SpectraMaxVariableWave
	if(value==1)
		wv[451]=1
	elseif(value==0)
		wv[451]=0
	endif
End	

Static function F_PaneltobekilledlaterHook(s)
	STRUCT WMWinHookStruct &s
	variable handledIt=0
	strswitch(s.eventname)
		case "keyboard":
			switch(s.keycode)
				case 27:
					DoKillWithoutSetWavenames("cancel_button")
					handledIt=1
					break
				case 13:
					DoSetWavenamesAndKill("load_button")
					handledIt=1
					break
				case 3:
					DoSetWavenamesAndKill("load_button")
					handledIt=1
					break
			endswitch
	endswitch
	return handledIt
end

// unit list:
// ---------
//       Metre               = 1;
//       centimetre          = 2;
//       millimetre          = 3;
//       microns             = 4;
//       nanometres          = 5;
//       angstroms           = 6;
//       acm_1               = 10;
//       rcm_1               = 11;
//       hertz               = 20;
//       Terahertz           = 21;
//       pascal              = 30;
//       atmosphere          = 31;
//       mm_hg               = 32;
//       degre_celsius       = 40;
//       degre_farenheit     = 41;
//       degre_kelvin        = 42;
//       electron_volt       = 50;
//       coups_par_seconde   = 51;
//       coups               = 52;
//       diodes              = 60;
//       pixels              = 61;
//       nanoseconde         = 70;
//       microseconde        = 71;
//       milliseconde        = 72;
//       seconde             = 73;
//       microvolt           = 80;
//       millivolt           = 81;
//       volt                = 82;
//       microamps           = 83;
//       milliamps           = 84;
//       amps                = 85;
//       unite_absorbance    = 86;
//       unite_transmittance = 87;
//       unite_pourcent_transmittance= 88;
//       file_number         = 90;
//       file_spectre_number = 91;
//       spectrum_var        = 92;
//       file_var            = 93;
//       region_var          = 94;
//       bloc_var            = 95;
//       utilisateur_1       = 100;
//       utilisateur_2       = 101;
//       utilisateur_3       = 102;
//       utilisateur_4       = 103;
//       utilisateur_5       = 104;
//       utilisateur_6       = 105;
//       epsilon_reel        = 108;
//       epsilon_imaginaire  = 109;
//       PSI                 = 110;
//       delta               = 111;
//       degre_angle         = 112;
//       minute_angle        = 113;
//       seconde_angle       = 114;
//       radians             = 115;
//       grades              = 116;
//       datapoint           = 117;